@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import numbers
 import operator
 from collections.abc import Iterable, Mapping, MutableMapping
 from functools import partial
@@ -22,9 +23,7 @@ from typing import Any, Callable, Generic, Optional, TypeVar, Union
 import numpy as np
 import pandas as pd
 import pyarrow as pa
-from packaging import version
 
-from .. import config
 from ..features import Features
 from ..features.features import _ArrayXDExtensionType, _is_zero_copy_only, decode_nested_example, pandas_types_mapper
 from ..table import Table
@@ -225,10 +224,14 @@ class PythonFeaturesDecoder:
         return self.features.decode_example(row, token_per_repo_id=self.token_per_repo_id) if self.features else row
 
     def decode_column(self, column: list, column_name: str) -> list:
-        return self.features.decode_column(column, column_name) if self.features else column
+        return (
+            self.features.decode_column(column, column_name, token_per_repo_id=self.token_per_repo_id)
+            if self.features
+            else column
+        )
 
     def decode_batch(self, batch: dict) -> dict:
-        return self.features.decode_batch(batch) if self.features else batch
+        return self.features.decode_batch(batch, token_per_repo_id=self.token_per_repo_id) if self.features else batch
 
 
 class PandasFeaturesDecoder:
@@ -304,49 +307,46 @@ class LazyDict(MutableMapping):
         self._format_all()
         return repr(self.data)
 
-    if config.PY_VERSION >= version.parse("3.9"):
-        # merging with the union ("|") operator is supported in Python 3.9+
+    def __or__(self, other):
+        if isinstance(other, LazyDict):
+            inst = self.copy()
+            other = other.copy()
+            other._format_all()
+            inst.keys_to_format -= other.data.keys()
+            inst.data = inst.data | other.data
+            return inst
+        if isinstance(other, dict):
+            inst = self.copy()
+            inst.keys_to_format -= other.keys()
+            inst.data = inst.data | other
+            return inst
+        return NotImplemented
 
-        def __or__(self, other):
-            if isinstance(other, LazyDict):
-                inst = self.copy()
-                other = other.copy()
-                other._format_all()
-                inst.keys_to_format -= other.data.keys()
-                inst.data = inst.data | other.data
-                return inst
-            if isinstance(other, dict):
-                inst = self.copy()
-                inst.keys_to_format -= other.keys()
-                inst.data = inst.data | other
-                return inst
-            return NotImplemented
+    def __ror__(self, other):
+        if isinstance(other, LazyDict):
+            inst = self.copy()
+            other = other.copy()
+            other._format_all()
+            inst.keys_to_format -= other.data.keys()
+            inst.data = other.data | inst.data
+            return inst
+        if isinstance(other, dict):
+            inst = self.copy()
+            inst.keys_to_format -= other.keys()
+            inst.data = other | inst.data
+            return inst
+        return NotImplemented
 
-        def __ror__(self, other):
-            if isinstance(other, LazyDict):
-                inst = self.copy()
-                other = other.copy()
-                other._format_all()
-                inst.keys_to_format -= other.data.keys()
-                inst.data = other.data | inst.data
-                return inst
-            if isinstance(other, dict):
-                inst = self.copy()
-                inst.keys_to_format -= other.keys()
-                inst.data = other | inst.data
-                return inst
-            return NotImplemented
-
-        def __ior__(self, other):
-            if isinstance(other, LazyDict):
-                other = other.copy()
-                other._format_all()
-                self.keys_to_format -= other.data.keys()
-                self.data |= other.data
-            else:
-                self.keys_to_format -= other.keys()
-                self.data |= other
-            return self
+    def __ior__(self, other):
+        if isinstance(other, LazyDict):
+            other = other.copy()
+            other._format_all()
+            self.keys_to_format -= other.data.keys()
+            self.data |= other.data
+        else:
+            self.keys_to_format -= other.keys()
+            self.data |= other
+        return self
 
     def __copy__(self):
         # Identical to `UserDict.__copy__`
@@ -499,7 +499,7 @@ class CustomFormatter(Formatter[dict, ColumnFormat, dict]):
     The transform must take as input a batch of data extracted for an arrow table using the python extractor,
     and return a batch.
     If the output batch is not a dict, then output_all_columns won't work.
-    If the ouput batch has several fields, then querying a single column won't work since we don't know which field
+    If the output batch has several fields, then querying a single column won't work since we don't know which field
     to return.
     """
 
@@ -566,7 +566,7 @@ def _check_valid_index_key(key: Union[int, slice, range, Iterable], size: int) -
 
 
 def key_to_query_type(key: Union[int, slice, range, str, Iterable]) -> str:
-    if isinstance(key, int):
+    if isinstance(key, numbers.Integral):
         return "row"
     elif isinstance(key, str):
         return "column"
