@@ -34,6 +34,7 @@ class SplitInfo:
     num_bytes: int = dataclasses.field(default=0, metadata={"include_in_asdict_even_if_is_default": True})
     num_examples: int = dataclasses.field(default=0, metadata={"include_in_asdict_even_if_is_default": True})
     shard_lengths: Optional[list[int]] = None
+    original_shard_lengths: Optional[list[int]] = None
 
     # Deprecated
     # For backward compatibility, this field needs to always be included in files like
@@ -516,7 +517,7 @@ class SplitReadInstruction:
         return list(self._splits.values())
 
 
-class SplitDict(dict):
+class SplitDict(dict[str, SplitInfo]):
     """Split info object."""
 
     def __init__(self, *args, dataset_name=None, **kwargs):
@@ -585,9 +586,10 @@ class SplitDict(dict):
 
     def _to_yaml_list(self) -> list:
         out = [asdict(s) for s in self.to_split_dict()]
-        # we don't need the shard lengths in YAML, since it depends on max_shard_size and num_proc
+        # we don't need the shard lengths in YAML
         for split_info_dict in out:
             split_info_dict.pop("shard_lengths", None)
+            split_info_dict.pop("original_shard_lengths", None)
         # we don't need the dataset_name attribute that is deprecated
         for split_info_dict in out:
             split_info_dict.pop("dataset_name", None)
